// game.js for Perlenspiel 3.1

/*
 Saga, Crushed by Kevin Harris. Created for the 2014 Global Game Jam.
 */

/*
Perlenspiel is a scheme by Professor Moriarty (bmoriarty@wpi.edu).
Perlenspiel is Copyright © 2009-14 Worcester Polytechnic Institute.
This file is part of Perlenspiel.

Perlenspiel is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published
by the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Perlenspiel is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You may have received a copy of the GNU Lesser General Public License
along with Perlenspiel. If not, see <http://www.gnu.org/licenses/>.
*/

// The following comment lines are for JSLint. Don't remove them!

/*jslint nomen: true, white: true */
/*global PS */

// This is a template for creating new Perlenspiel games

// All of the functions below MUST exist, or the engine will complain!

// PS.init( system, options )
// Initializes the game
// This function should normally begin with a call to PS.gridSize( x, y )
// where x and y are the desired initial dimensions of the grid
// [system] = an object containing engine and platform information; see documentation for details
// [options] = an object with optional parameters; see documentation for details

PS.init = function( system, options ) {
	"use strict";

	// Use PS.gridSize( x, y ) to set the grid to
	// the initial dimensions you want (32 x 32 maximum)
	// Do this FIRST to avoid problems!
	// Otherwise you will get the default 8x8 grid

	PS.gridSize( 8, 8 );
	PS.border( PS.ALL, PS.ALL, 1 );
	PS.borderColor( PS.ALL, PS.ALL, PS.COLOR_BLACK );

	borderIn();
	nextNote();

	// Add any other initialization code you need here
	PS.keyRepeat( false );
	document.body.style.backgroundColor = "#333333";

	newGame()
};

// PS.touch ( x, y, data, options )
// Called when the mouse button is clicked on a bead, or when a bead is touched
// It doesn't have to do anything
// [x] = zero-based x-position of the bead on the grid
// [y] = zero-based y-position of the bead on the grid
// [data] = the data value associated with this bead, 0 if none has been set
// [options] = an object with optional parameters; see documentation for details

PS.touch = function( x, y, data, options ) {
	"use strict";

	// Uncomment the following line to inspect parameters
	// PS.debug( "PS.touch() @ " + x + ", " + y + "\n" );

	// Add code here for mouse clicks/touches over a bead
};

// PS.release ( x, y, data, options )
// Called when the mouse button is released over a bead, or when a touch is lifted off a bead
// It doesn't have to do anything
// [x] = zero-based x-position of the bead on the grid
// [y] = zero-based y-position of the bead on the grid
// [data] = the data value associated with this bead, 0 if none has been set
// [options] = an object with optional parameters; see documentation for details

PS.release = function( x, y, data, options ) {
	"use strict";

	// Uncomment the following line to inspect parameters
	// PS.debug( "PS.release() @ " + x + ", " + y + "\n" );

	// Add code here for when the mouse button/touch is released over a bead
};

// PS.enter ( x, y, button, data, options )
// Called when the mouse/touch enters a bead
// It doesn't have to do anything
// [x] = zero-based x-position of the bead on the grid
// [y] = zero-based y-position of the bead on the grid
// [data] = the data value associated with this bead, 0 if none has been set
// [options] = an object with optional parameters; see documentation for details

PS.enter = function( x, y, data, options ) {
	"use strict";

	// Uncomment the following line to inspect parameters
	// PS.debug( "PS.enter() @ " + x + ", " + y + "\n" );

	// Add code here for when the mouse cursor/touch enters a bead
};

// PS.exit ( x, y, data, options )
// Called when the mouse cursor/touch exits a bead
// It doesn't have to do anything
// [x] = zero-based x-position of the bead on the grid
// [y] = zero-based y-position of the bead on the grid
// [data] = the data value associated with this bead, 0 if none has been set
// [options] = an object with optional parameters; see documentation for details

PS.exit = function( x, y, data, options ) {
	"use strict";

	// Uncomment the following line to inspect parameters
	// PS.debug( "PS.exit() @ " + x + ", " + y + "\n" );

	// Add code here for when the mouse cursor/touch exits a bead
};

// PS.exitGrid ( options )
// Called when the mouse cursor/touch exits the grid perimeter
// It doesn't have to do anything
// [options] = an object with optional parameters; see documentation for details

PS.exitGrid = function( options ) {
	"use strict";

	// Uncomment the following line to verify operation
	// PS.debug( "PS.exitGrid() called\n" );

	// Add code here for when the mouse cursor/touch moves off the grid
};

// PS.keyDown ( key, shift, ctrl, options )
// Called when a key on the keyboard is pressed
// It doesn't have to do anything
// [key] = ASCII code of the pressed key, or one of the following constants:
// Arrow keys = PS.ARROW_UP, PS.ARROW_DOWN, PS.ARROW_LEFT, PS.ARROW_RIGHT
// Function keys = PS.F1 through PS.F1
// [shift] = true if shift key is held down, else false
// [ctrl] = true if control key is held down, else false
// [options] = an object with optional parameters; see documentation for details

var ignoreKeyUp = false;
PS.keyDown = function( key, shift, ctrl, options ) {
	"use strict";

	if (!acceptInput) {
		ignoreKeyUp = true;
		return;
	}
	ignoreKeyUp = false;
	// Uncomment the following line to inspect parameters
	// PS.debug( "DOWN: key = " + key + ", shift = " + shift + "\n" );

	// Add code here for when a key is pressed
	if ( key == PS.KEY_ARROW_UP ) {
		playerPushStart();
	}
	else if ( key == 32 ) {
		playerPrepareStart();
	}
};

// PS.keyUp ( key, shift, ctrl, options )
// Called when a key on the keyboard is released
// It doesn't have to do anything
// [key] = ASCII code of the pressed key, or one of the following constants:
// Arrow keys = PS.ARROW_UP, PS.ARROW_DOWN, PS.ARROW_LEFT, PS.ARROW_RIGHT
// Function keys = PS.F1 through PS.F12
// [shift] = true if shift key is held down, false otherwise
// [ctrl] = true if control key is held down, false otherwise
// [options] = an object with optional parameters; see documentation for details

PS.keyUp = function( key, shift, ctrl, options ) {
	"use strict";

	if (!acceptInput || ignoreKeyUp) return;
	// Uncomment the following line to inspect parameters
	// PS.debug( "PS.keyUp(): key = " + key + ", shift = " + shift + ", ctrl = " + ctrl + "\n" );

	// Add code here for when a key is released
	if ( key == PS.KEY_ARROW_UP ) {
		playerPushDone();
	}
	else if ( key == 32 ) {
		playerPrepareDone();
	}
};

// PS.input ( sensors, options )
// Called when an input device event (other than mouse/touch/keyboard) is detected
// It doesn't have to do anything
// [sensors] = an object with sensor information; see documentation for details
// [options] = an object with optional parameters; see documentation for details

PS.input = function( sensors, options ) {
	"use strict";

	// Uncomment the following block to inspect parameters
	/*
	PS.debug( "PS.input() called\n" );
	var device = sensors.wheel; // check for scroll wheel
	if ( device )
	{
		PS.debug( "sensors.wheel = " + device + "\n" );
	}
	*/
	
	// Add code here for when an input event is detected
};

candyTexture = [
	PS.COLOR_WHITE, PS.COLOR_RED, PS.COLOR_RED, PS.COLOR_RED,
	PS.COLOR_RED, PS.COLOR_WHITE, PS.COLOR_RED, PS.COLOR_RED,
	PS.COLOR_RED, PS.COLOR_RED, PS.COLOR_WHITE, PS.COLOR_RED,
	PS.COLOR_RED, PS.COLOR_RED, PS.COLOR_RED, PS.COLOR_WHITE
];

terrainColors = [
	PS.COLOR_BLACK, PS.COLOR_GRAY_DARK, 
	PS.COLOR_GRAY, PS.COLOR_GRAY_LIGHT
];

playerDefaultColors = [
	PS.makeRGB( 255, 13, 108 ),
	PS.makeRGB( 232, 12, 209 ),
	PS.makeRGB( 200, 0, 255 ),
	PS.makeRGB( 124, 12, 232 ),
	PS.makeRGB( 71, 13, 255 )
];

playerPrepareColors = [
	PS.makeRGB( 255, 130, 188 ),
	PS.makeRGB( 232, 118, 228 ),
	PS.makeRGB( 226, 143, 255 ),
	PS.makeRGB( 172, 118, 232 ),
	PS.makeRGB( 155, 130, 255 )
];

playerPushColors = [
	PS.makeRGB( 255, 205, 235 ),
	PS.makeRGB( 229, 186, 232 ),
	PS.makeRGB( 243, 217, 255 ),
	PS.makeRGB( 204, 186, 232 ),
	PS.makeRGB( 211, 205, 255 )
];

bassShort = [
	PS.harpsichord( 9, false ), PS.harpsichord( 12, false ),
	PS.harpsichord( 14, false ), PS.harpsichord( 16, false ),
	PS.harpsichord( 19, false )
];

melodyShort = [
	PS.harpsichord(21, true), PS.harpsichord(24, true),
	PS.harpsichord(26, true), PS.harpsichord(28, true),
	PS.harpsichord(31, true),

	PS.harpsichord(33, true), PS.harpsichord(36, true),
	PS.harpsichord(38, true), PS.harpsichord(40, true),
	PS.harpsichord(43, true)
];

var melodyIndex = 0;
var candyRow;
var stepsTaken;
var prepSteps;
var msgIndex;
var lastFive = 0;
var terrainSeed;
var acceptInput;

var playerDefaultIndex;
var playerPrepareIndex;
var playerPushIndex;

var messages = [
	"UP: heave",
	"SPACE: ho  UP: heave",
	"ho ho heave",
	"ho ho ho heave…"
];

var failMessages = [
	 "my burden falls; I follow",
	 "to fallen candy I go",
	 "my task renewed",
	 "I pause, then descend",
	 "I retread familiar ground"
];

var phraseFive = [
	"summer grasses",
	"the bright autumn moon",
	"white crests",
	"the rising sun",
	"scent of plum blossoms",
	"scent of cherry blossoms",
	"travelling at dusk",
	"an old pond",
	"the sound of water",
	"from all directions",
	"dewdrops falling",
	"scent of rain",
	"a pine tree, tilted",
	"the waterfall glistens",
	"crickets chirp",
	"a banana tree",
	"without a hat",
	"cold rain falls",
	"small mosquitoes",
	"cicadas chirp",
	"deep in mourning"
];

var phraseSeven = [
	"all that remains",
	"sea mist upon my gate",
	"a path through misty mountains",
	"alone upon the path",
	"a frog jumps",
	"winds bearing petals",
	"if only I could wash",
	"vanishing before me",
	"my dreams wander",
	"my thoughts wander",
	"a butterfly's wing, torn",
	"an field, empty and brown",
	"my voice weeps",
	"swaying without falling",
	"with all other things",
	"a doll upon the ground",
	"a caterpillar hangs",
	"lice and fleas bite",
	"the clouded sky unfolds"
];

function borderIn( ) {
	PS.borderFade( PS.ALL, PS.ALL, 60 * 5 );
	PS.borderColor( PS.ALL, PS.ALL, PS.COLOR_WHITE );
	PS.timerStart( 60 * 7, borderOut );

	return PS.ERROR;
}

function borderOut( ) {
	PS.borderFade( PS.ALL, PS.ALL, 60 * 11 );
	PS.borderColor( PS.ALL, PS.ALL, PS.COLOR_BLACK );
	PS.timerStart( 60 * 12, borderIn );

	return PS.ERROR;
}

function nextNote( ) {
	if (PS.random(100) < 25) {
		melodyIndex = (melodyIndex + PS.random(melodyShort.length * 2) - melodyShort.length - 1);
	}
	else {
		melodyIndex += PS.random(3) - 2;
	}
	melodyIndex = melodyIndex < 0 ? melodyIndex + melodyShort.length : melodyIndex % melodyShort.length;

	PS.audioPlay(melodyShort[melodyIndex]);
	PS.timerStart(60 * PS.random(4), nextNote);

	return PS.ERROR;
}

function newCandy() {
	candyRow = 2;
	for (i = 0; i < 16; ++i) {
		candyTexture[i] = PS.COLOR_RED;
	}

	var r0 = PS.random( 4 ) - 1;
	var r1 = PS.random( 4 ) - 1;
	// BUG: No pathmap required.
	var points = PS.line(0, r0, 3, r1);

	candyTexture[r0 * 4] = PS.COLOR_WHITE;
	for (i = 0; i < 3; ++i) {
		var point = points[i];
		candyTexture[point[1] * 4 + point[0]] = PS.COLOR_WHITE;
	}
}

function drawCandy() {
	for (c = 3; c < 5; ++c) {
		if (candyRow < 8) PS.color( c, candyRow, candyTexture[c - 2] );
		if (candyRow + 3 < 8) PS.color( c, candyRow + 3, candyTexture[c + 10] );
	}

	for (c = 2; c < 6; ++c) {
		if (candyRow + 1 < 8) PS.color( c, candyRow + 1, candyTexture[c + 2] );
		if (candyRow + 2 < 8) PS.color( c, candyRow + 2, candyTexture[c + 6] );
	}
}

function clearCandy( ) {
	// BUG: PS.COLOR_DARK_GRAY fails
	//PS.color( PS.ALL, PS.ALL, 4210752 );
	PS.color( PS.ALL, PS.ALL, terrainColors[terrainSeed] );
}

rollCandy = function( ) {
	clearCandy();
	playerDefault();
	--candyRow;

	var t0 = candyTexture[0];
	var t1 = candyTexture[1];
	var t2 = candyTexture[2];
	var t3 = candyTexture[3];

	for (r = 0; r < 3; ++r) {
		candyTexture[4 * r + 0] = candyTexture[4 * (r + 1) + 0];
		candyTexture[4 * r + 1] = candyTexture[4 * (r + 1) + 1];
		candyTexture[4 * r + 2] = candyTexture[4 * (r + 1) + 2];
		candyTexture[4 * r + 3] = candyTexture[4 * (r + 1) + 3];
	}

	candyTexture[12] = t0;
	candyTexture[13] = t1;
	candyTexture[14] = t2;
	candyTexture[15] = t3;

	++msgIndex;
	if (msgIndex < messages.length) {
		PS.statusText( messages[msgIndex] );
	}
	else if (msgIndex < messages.length + 2) PS.statusText( "" );
	else {
		var haikuIndex = (msgIndex - messages.length - 2) % 4;
		if (haikuIndex == 1) {
			PS.statusText( phraseSeven[PS.random(phraseSeven.length) - 1] );
		}
		else if (haikuIndex == 0 || haikuIndex == 2) {
			var nextFive = lastFive;
			while (nextFive == lastFive) {
				nextFive = PS.random(phraseFive.length) - 1;
			}
			PS.statusText( phraseFive[nextFive] );
			lastFive = nextFive;
		}
		else PS.statusText ( "" );
	}
	drawCandy();
	PS.timerStart( 10, pushedUp );
}

function fallCandy( ) {
	clearCandy();
	++candyRow;

	var t0 = candyTexture[12];
	var t1 = candyTexture[13];
	var t2 = candyTexture[14];
	var t3 = candyTexture[15];

	for (r = 2; r >= 0; --r) {
		candyTexture[4 * (r + 1) + 0] = candyTexture[4 * r + 0];
		candyTexture[4 * (r + 1) + 1] = candyTexture[4 * r + 1];
		candyTexture[4 * (r + 1) + 2] = candyTexture[4 * r + 2];
		candyTexture[4 * (r + 1) + 3] = candyTexture[4 * r + 3];
	}

	candyTexture[0] = t0;
	candyTexture[1] = t1;
	candyTexture[2] = t2;
	candyTexture[3] = t3;

	drawCandy();

	if (candyRow > 8) {
		PS.fade( PS.ALL, PS.ALL, 60 );
		PS.color( PS.ALL, PS.ALL, PS.COLOR_BLACK );
		PS.timerStart( 70, newGame );
		
		return PS.ERROR;
	}
	return true
}

function newGame( ) { 
	PS.fade( PS.ALL, PS.ALL, PS.DEFAULT );
	terrainSeed = PS.random(terrainColors.length) - 1;
	clearCandy();
	
	playerDefaultIndex = PS.random(playerDefaultColors.length) - 1;
	playerPrepareIndex = PS.random(playerPrepareColors.length) - 1;
	playerPushIndex = PS.random(playerPushColors.length) - 1;

	playerDefault();

	PS.fade( 3, 6, 5 );
	PS.fade( 4, 6, 5 );

	msgIndex = prepSteps = stepsTaken = 0;
	PS.statusText( messages[msgIndex] );

	newCandy();
	drawCandy();

	acceptInput = true;
	return PS.ERROR;
}

function pushedUp( ) {
	clearCandy();
	++candyRow;
	drawCandy();
	playerDefault();

	++stepsTaken;
	acceptInput = true;
	return PS.ERROR;
}

function playerDefault( ) {
	var color = playerDefaultColors[playerDefaultIndex];
	PS.color( 3, 6, color );
	PS.color( 4, 6, color );
}

function playerPrepareStart( ) {
	var color = playerPrepareColors[playerPrepareIndex];
	PS.color( 3, 6, color );
	PS.color( 4, 6, color );

	PS.audioPlay( bassShort[PS.random(bassShort.length) - 1] );
}

function playerPrepareDone( ) {
	playerDefault();
	++prepSteps;

	PS.audioPlay( bassShort[PS.random(bassShort.length) - 1] );
}

function playerPushStart( ) {
	var color = playerPushColors[playerPushIndex];
	PS.color( 3, 6, color );
	PS.color( 4, 6, color );
	PS.audioPlay( bassShort[PS.random(bassShort.length) - 1] );
}

function playerPushDone( ) {
	playerDefault();
	acceptInput = false;
	if (prepSteps == stepsTaken) {
		rollCandy();

		PS.audioPlay( bassShort[PS.random(bassShort.length) - 1] );
	}
	else {
		PS.fade( 3, 6, PS.DEFAULT );
		PS.fade( 4, 6, PS.DEFAULT );

		var failure = PS.random(failMessages.length) - 1;
		PS.statusText( failMessages[failure] );
		PS.timerStart( 15, fallCandy );

		PS.audioPlay( bassShort[PS.random(2) - 1] );
	}

	prepSteps = 0;
}

